import { system } from "@minecraft/server";
import { addCoords, displayActionBar, rotateVector } from "../../utils/utils";
import { arrowRegistry } from "../registry";
import { getAbilityName } from "../../utils/addonUtils";

export const arrow_shield_ability = {
  key: "arrow_shield",
  unlock_level: 30,
  enableTitle: true,
  description: (level) => {
    return [`${getDuration(level)}`];
  },
  cooldown: (level) => getCooldown(level),
  onEnable: (data) => {
    const { player, skill, level } = data;

    player.addTag("pod_rpg.disableMovement");
    player.runCommand("inputpermission set @s movement disabled");

    const baseVector = { x: 0.25, y: 0.3, z: 0 };
    const POINTS = 25;
    let i = 0;
    const runId = system.runInterval(async () => {
      if (i % 3 === 0) {
        displayActionBar(player, 2, {
          rawtext: [
            { text: "§p" },
            ...getAbilityName(arrow_shield_ability),
            {
              text: ` ${(getDuration(level) - i / 20).toFixed(1)}s`,
            },
          ],
        });
        for (const entity of player.dimension.getEntities({
          location: player.location,
          maxDistance: 3,
          families: ["monster"],
        })) {
          entity.applyKnockback(
            entity.location.x - player.location.x,
            entity.location.z - player.location.z,
            1,
            0.3
          );
        }
      }
      if (i === getDuration(level) * 20 - 1) {
        displayActionBar(player, 2, "§r");
        if (
          player.runCommand("inputpermission set @s movement enabled")
            .successCount !== 0
        ) {
          player.removeTag("pod_rpg.disableMovement");
        }
        system.clearRun(runId);
      }
      i++;

      player.applyKnockback(0, 0, 0, 0);

      const arrow = player.dimension.spawnEntity(
        "minecraft:arrow",
        addCoords(player.location, { x: 0, y: 2.5, z: 0 })
      );
      arrow.applyImpulse(rotateVector(baseVector, (360 / POINTS) * i));
      arrowRegistry[arrow.id] = {
        arrow: arrow,
        player: player,
        onImpact: handleImpact,
      };
    });
    skill.abilityCooldown.set(player, getCooldown(level) * 20, skill);
  },
};

const handleImpact = (arrow, event, type) => {
  if (type === "block") arrow.arrow.remove();
};

const getDuration = (level) => {
  let value = 5;
  if (level >= 1) value += 2.5;
  if (level >= 3) value += 2.5;
  return value;
};

const getCooldown = (level) => {
  let cooldown = 60;
  if (level >= 2) cooldown -= 5;
  if (level >= 4) cooldown -= 5;
  if (level >= 5) cooldown -= 10;
  return cooldown;
};
