import { system } from "@minecraft/server";
import { getAbilityLevel } from "../../utils/addonUtils";
import { getSetting } from "../../utils/settings";
import { arrowRegistry } from "../registry";
import { archer_skill } from "../skills/archer";
import { spawnParticle } from "../../utils/utils";

export const explosive_arrow_ability = {
  key: "explosive_arrow",
  unlock_level: 15,
  enableTitle: false,
  description: () => {
    return [];
  },
  cooldown: (level) => getCooldown(level),
  onEnable: (event) => {
    const level = getAbilityLevel(
      event.player,
      archer_skill,
      explosive_arrow_ability
    );
    arrowRegistry[event.arrow.id] = {
      arrow: event.arrow,
      source: event.player,
      onImpact: handleImpact,
    };
    archer_skill.abilityCooldown.set(
      event.player,
      getCooldown(level) * 20,
      archer_skill
    );

    if (getSetting("particles")?.valueOf() === true) {
      const runId = system.runInterval(() => {
        if (!event.arrow.isValid()) {
          system.clearRun(runId);
          return;
        }
        spawnParticle(
          event.arrow.dimension,
          event.arrow.location,
          "pod_rpg:explosivearrow"
        );
      });
    }
  },
};

const handleImpact = (arrow, event, type) => {
  const level = getAbilityLevel(
    arrow.source,
    archer_skill,
    explosive_arrow_ability
  );
  const explosion = arrow.source.dimension.spawnEntity(
    "pod_rpg:explosion",
    event.location
  );
  explosion.triggerEvent(
    `pod_rpg:kaboom${level >= 4 ? 2 : level >= 2 ? 1 : 0}${
      getSetting("explosion_damage")?.valueOf() === true ? "t" : "f"
    }`
  );
  if (type === "block") arrow.arrow.remove();
};

const getCooldown = (level) => {
  let cooldown = 60;
  if (level >= 1) cooldown -= 5;
  if (level >= 3) cooldown -= 5;
  if (level >= 5) cooldown -= 10;
  return cooldown;
};
