import { getAbilityLevel } from "../../utils/addonUtils";
import { getSetting } from "../../utils/settings";
import { rotateVector, spawnParticle } from "../../utils/utils";
import { arrowRegistry } from "../registry";
import { archer_skill } from "../skills/archer";

export const triple_shot_ability = {
  key: "triple_shot",
  unlock_level: 5,
  enableTitle: false,
  description: () => {
    return [];
  },
  cooldown: (level) => getCooldown(level),
  onEnable: (event) => {
    const { arrow, player } = event;

    spawnParticle(arrow.dimension, arrow.location, "pod_rpg.tripleshot");

    const level = getAbilityLevel(player, archer_skill, triple_shot_ability);

    const rArrow = arrow.dimension.spawnEntity(
      "minecraft:arrow",
      arrow.location
    );
    const lArrow = arrow.dimension.spawnEntity(
      "minecraft:arrow",
      arrow.location
    );

    arrowRegistry[rArrow.id] = {
      arrow: rArrow,
      source: player,
      onImpact: handleImpact,
    };
    arrowRegistry[lArrow.id] = {
      arrow: lArrow,
      source: player,
      onImpact: handleImpact,
    };
    arrowRegistry[arrow.id] = {
      arrow: arrow,
      source: player,
      onImpact: handleImpact,
    };

    rArrow.applyImpulse(rotateVector(arrow.getVelocity(), 10));

    lArrow.applyImpulse(rotateVector(arrow.getVelocity(), -10));

    archer_skill.abilityCooldown.set(
      player,
      getCooldown(level) * 20,
      archer_skill
    );
  },
};

const handleImpact = (arrow, event, type) => {
  if (getAbilityLevel(arrow.source, archer_skill, triple_shot_ability) < 5)
    return;
  const explosion = arrow.source.dimension.spawnEntity(
    "pod_rpg:explosion",
    event.location
  );
  explosion.triggerEvent(
    `pod_rpg:arrow0${
      getSetting("explosion_damage")?.valueOf() === true ? "t" : "f"
    }`
  );
  if (type === "block") arrow.arrow.remove();
};

const getCooldown = (level) => {
  let cooldown = 60;
  if (level >= 1) cooldown -= 5;
  if (level >= 2) cooldown -= 10;
  if (level >= 3) cooldown -= 5;
  if (level >= 4) cooldown -= 10;
  return cooldown;
};
