import { system, world } from "@minecraft/server";
import { spawnParticle } from "../../utils/utils";

const DELAY = 80;

export const tnt_boost_ability = {
  key: "tnt_boost",
  unlock_level: 30,
  enableTitle: true,
  description: () => {
    return [];
  },
  cooldown: (level) => getCooldown(level),
  onEnable: (data) => {
    const { player, skill, level } = data;
    const barrel = player.dimension.spawnEntity(
      "pod_rpg:tnt_barrel",
      player.location
    );
    barrel.runCommand(`ride "${player.name}" start_riding @s teleport_rider`);
    player.addTag("pod_rpg.disableMovement");
    player.addTag("pod_rpg.customCamera");
    player.runCommand("inputpermission set @s movement disabled");
    player.runCommand("camera @s set minecraft:third_person");
    player.runCommand(`camerashake add @s 0.75 ${DELAY / 20} positional`);
    world.playSound("fire.ignite", barrel.location);
    world.playSound("random.fuse", barrel.location);
    let ticks = 0;
    const runId = system.runInterval(() => {
      ticks++;
      if (0.3 >= Math.random() && barrel.isValid()) {
        world.playSound("mob.zombie.wood", barrel.location, {
          volume: Math.max(Math.random(), 0.3),
          pitch: Math.max(Math.random(), 0.5),
        });
      }
      switch (ticks) {
        case DELAY:
          world.playSound("random.explode", barrel.location);
          spawnParticle(
            player.dimension,
            barrel.location,
            "minecraft:huge_explosion_emitter"
          );
          barrel.remove();

          if (
            player.runCommand("inputpermission set @s movement enabled")
              .successCount !== 0
          ) {
            player.removeTag("pod_rpg.disableMovement");
          }
          if (player.runCommand("camera @s clear").successCount !== 0) {
            player.removeTag("pod_rpg.customCamera");
          }
          break;
        case DELAY + 2:
          player.applyKnockback(0, 0, 0, level >= 4 ? 8 : level >= 2 ? 6 : 4);
          player.addTag("pod_rpg.slowFalling");
          player.addEffect("slow_falling", 20000000, {
            showParticles: false,
            amplifier: 2,
          });
          system.clearRun(runId);
          break;
      }
    });
    skill.abilityCooldown.set(player, getCooldown(level) * 20, skill);
  },
};

const getCooldown = (level) => {
  let cooldown = 120;
  if (level >= 1) cooldown -= 10;
  if (level >= 3) cooldown -= 10;
  if (level >= 5) cooldown -= 10;
  return cooldown;
};
