import { system } from "@minecraft/server";
import { displayActionBar, hasItem, spawnParticle } from "../../utils/utils";

export const homecoming_ability = {
  key: "homecoming",
  unlock_level: 30,
  enableTitle: false,
  description: (level) => {
    return [`${getCost(level)}`, `${getDuration(level)}`];
  },
  onEnable: (data) => {
    const { player, skill, level } = data;
    if (hasPearls(player, getCost(level))) {
      if (!player.getSpawnPoint()) {
        player.sendMessage({
          rawtext: [
            { translate: "pod_rpg.abilities.homecoming.cancel.spawnpoint" },
          ],
        });
        return;
      }
      const DELAY = 20 * 15;
      skill.abilityCooldown.set(player, DELAY, skill);
      let ticksPassed = 0;
      player.runCommand("fog @s push pod_rpg:homecoming pod_rpg_homecoming");
      const runId = system.runInterval(() => {
        const v = player.getVelocity();
        if (
          ticksPassed > 10 &&
          Math.sqrt(v.x ** 2 + v.y ** 2, v.z ** 2) !== 0
        ) {
          player.sendMessage({
            rawtext: [
              { translate: "pod_rpg.abilities.homecoming.cancel.move" },
            ],
          });
          player.runCommand("fog @s remove pod_rpg_homecoming");
          skill.abilityCooldown.set(player, 0, skill);
          system.clearRun(runId);
          return;
        }
        if (ticksPassed % Math.floor(10 + Math.random() * 10) === 0) {
          player.playSound("ambient.weather.thunder", {
            volume: ticksPassed / DELAY,
            pitch: 1.3 - ticksPassed / DELAY,
          });
        }
        if (ticksPassed % 10 === 0)
          spawnParticle(
            player.dimension,
            player.location,
            "pod_rpg:homecoming"
          );

        let spawnpoint;
        switch (ticksPassed) {
          case DELAY - 80:
            player.playSound("portal.trigger");
            break;
          case DELAY - 20:
            player.addEffect("blindness", 40, {
              amplifier: 10,
              showParticles: false,
            });
            break;
          case DELAY:
            system.clearRun(runId);
            player.runCommand("fog @s pop pod_rpg_homecoming");
            spawnpoint = player.getSpawnPoint();
            if (!spawnpoint) {
              player.sendMessage({
                rawtext: [
                  {
                    translate: "pod_rpg.abilities.homecoming.cancel.spawnpoint",
                  },
                ],
              });
              return;
            }
            if (!hasPearls(player, getCost(level))) return;

            player.addEffect("slowness", getDuration(level) * 20, {
              amplifier: 1,
              showParticles: false,
            });
            player.teleport(player.getSpawnPoint(), {
              dimension: player.getSpawnPoint().dimension,
            });
            player.runCommand(`clear @s ender_pearl 0 ${getCost(level)}`);
            displayActionBar(player, 0, "§r");
            return;
        }
        displayActionBar(player, 0, {
          rawtext: [
            {
              translate: "pod_rpg.abilities.homecoming.teleportation_in",
              with: [`${((DELAY - ticksPassed) / 20).toFixed(1)}`],
            },
          ],
        });

        ticksPassed++;
      });
    }
  },
};

const hasPearls = (player, amount) => {
  const result = hasItem(player, "minecraft:ender_pearl", amount);
  if (!result)
    player.sendMessage({
      rawtext: [{ translate: "pod_rpg.abilities.homecoming.cancel.pearls" }],
    });
  return result;
};

const getCost = (level) => {
  let value = 5;
  if (level >= 3) value -= 1;
  if (level >= 5) value -= 1;
  return value;
};
const getDuration = (level) => {
  let value = 60;
  if (level >= 1) value -= 10;
  if (level >= 2) value -= 10;
  if (level >= 4) value -= 10;
  return value;
};
