import { BlockPermutation, system } from "@minecraft/server";
import { getAbilityName } from "../../utils/addonUtils";
import TimerUtil from "../../utils/cooldowns";
import { farming_rewards } from "../skills/farming";
import { pod_farm_crop_definitions } from "../pod_farm";
import {
  addCoords,
  indexOfPermutationMatched,
  isString,
  spawnParticle,
} from "../../utils/utils";
import { isAddonInstalled } from "../../main";

export const fertiliser_timer = new TimerUtil();

export const fertiliser_ability = {
  key: "fertiliser",
  unlock_level: 30,
  enableTitle: true,
  description: (level) => {
    return [`${getDuration(level)}`];
  },
  cooldown: (level) => getCooldown(level),
  onEnable: (data) => {
    const { player, skill, level } = data;
    fertiliser_timer.set(player.name, getDuration(level) * 20);
    fertiliser_timer.enableDisplay(player, () => {
      return {
        rawtext: [
          { text: "§p" },
          ...getAbilityName(fertiliser_ability),
          {
            text: ` ${(
              (fertiliser_timer.get(player.name) - system.currentTick) /
              20
            ).toFixed(1)}s`,
          },
        ],
      };
    });
    skill.abilityCooldown.set(player, getCooldown(level) * 20, skill);
  },
};

export const handleFertiliserAbility = async (dimension, location) => {
  let promises = [];
  for (let x = -1; x < 2; x++) {
    for (let z = -1; z < 2; z++) {
      promises.push(
        (async () => {
          const affectedBlock = dimension.getBlock(
            addCoords(location, { x: x, y: 0, z: z })
          );

          for (const entry of farming_rewards) {
            const i = indexOfPermutationMatched(
              affectedBlock.permutation,
              isString(entry.blocks[0])
                ? entry.blocks
                : entry.blocks.map((v) => {
                    return { block: v.block, states: {} };
                  })
            );
            if (i !== -1) {
              affectedBlock.setPermutation(
                BlockPermutation.resolve(
                  entry.blocks[i].block,
                  entry.blocks[i].states
                )
              );
              break;
            }
          }
        })()
      );
    }
  }
  await Promise.all(promises);
  spawnParticle(dimension, location, "pod_rpg:superfertilizer");
  //POD FARM INTEGRATION
  if (!isAddonInstalled("pod_farm")) return;
  const entities = dimension.getEntities({
    location: location,
    maxDistance: 1.5,
    families: ["pod_farm_crop"],
  });
  promises = [];
  for (const entity of entities) {
    promises.push(
      (async () => {
        const crop = pod_farm_crop_definitions.find(
          (v) => v.id === entity.typeId.replace("pod_farm:", "")
        );
        if (crop) entity.setProperty("pod_farm:growth", crop.max_growth);
      })()
    );
  }
  Promise.all(promises);
};

const getCooldown = (level) => {
  let cooldown = 90;
  if (level >= 1) cooldown -= 5;
  if (level >= 3) cooldown -= 5;
  if (level >= 5) cooldown -= 10;
  return cooldown;
};

const getDuration = (level) => {
  let duration = 5;
  if (level >= 2) duration += 5;
  if (level >= 4) duration += 5;
  return duration;
};
