import { system } from "@minecraft/server";
import { getAbilityName } from "../../utils/addonUtils";
import TimerUtil from "../../utils/cooldowns";
import { farming_rewards } from "../skills/farming";
import {
  addCoords,
  isString,
  locationString,
  permutationMatches,
} from "../../utils/utils";
import { isAddonInstalled } from "../../main";

export const harvest_timer = new TimerUtil();

export const harvest_ability = {
  key: "harvest",
  unlock_level: 15,
  enableTitle: true,
  description: (level) => {
    return [`${getDuration(level)}`];
  },
  cooldown: (level) => getCooldown(level),
  onEnable: (data) => {
    const { player, skill, level } = data;
    harvest_timer.set(player.name, getDuration(level) * 20);
    harvest_timer.enableDisplay(player, () => {
      return {
        rawtext: [
          { text: "§p" },
          ...getAbilityName(harvest_ability),
          {
            text: ` ${(
              (harvest_timer.get(player.name) - system.currentTick) /
              20
            ).toFixed(1)}s`,
          },
        ],
      };
    });
    skill.abilityCooldown.set(player, getCooldown(level) * 20, skill);
  },
};

export const handleHarvestAbility = async (dimension, location) => {
  const promises = [];
  for (let x = -2; x < 3; x++) {
    for (let z = -2; z < 3; z++) {
      promises.push(
        (async () => {
          if ((x !== z) !== 0) {
            const coords = addCoords(location, { x: x, y: 0, z: z });
            if (
              permutationMatches(
                dimension.getBlock(coords).permutation,
                farming_rewards.map((v) => {
                  return isString(v.blocks[0])
                    ? v.blocks[0]
                    : v.blocks[0].block;
                })
              )
            )
              dimension.runCommandAsync(
                `setblock ${locationString(coords)} air destroy`
              );
          }
        })()
      );
    }
  }
  //POD FARM INTEGRATION
  if (!isAddonInstalled("pod_farm")) return;
  for (const entity of dimension.getEntities({
    location: location,
    maxDistance: 2.85,
    families: ["pod_farm_crop"],
  })) {
    entity.triggerEvent("pod_farm:break");
  }
};

const getCooldown = (level) => {
  let cooldown = 90;
  if (level >= 1) cooldown -= 5;
  if (level >= 3) cooldown -= 5;
  if (level >= 5) cooldown -= 10;
  return cooldown;
};

const getDuration = (level) => {
  let duration = 5;
  if (level >= 2) duration += 5;
  if (level >= 4) duration += 5;
  return duration;
};
