import { EntityDamageCause, GameMode, system, world } from "@minecraft/server";
import { getAbilityName } from "../../utils/addonUtils";
import TimerUtil from "../../utils/cooldowns";
import {
  addCoords,
  getDirection,
  getDistance,
  spawnParticle,
} from "../../utils/utils";
import { getSetting } from "../../utils/settings";

export const chain_strike_timer = new TimerUtil();

export const chain_strike_ability = {
  key: "chain_strike",
  unlock_level: 15,
  enableTitle: true,
  description: (level) => {
    return [`${getDamageDecrease(level)}`, `${getDuration(level)}`];
  },
  cooldown: (level) => getCooldown(level),
  onEnable: (data) => {
    const { player, skill, level } = data;
    chain_strike_timer.set(player.name, getDuration(level) * 20);
    chain_strike_timer.enableDisplay(player, () => {
      return {
        rawtext: [
          { text: "§p" },
          ...getAbilityName(chain_strike_ability),
          {
            text: ` ${(
              (chain_strike_timer.get(player.name) - system.currentTick) /
              20
            ).toFixed(1)}s`,
          },
        ],
      };
    });
    skill.abilityCooldown.set(player, getCooldown(level) * 20, skill);
  },
};

export const handleChainStrike = (event, level) => {
  const { damageSource, damage, hurtEntity } = event;
  if (damageSource.cause === "projectile") return;
  if (damageSource.cause === EntityDamageCause.magic) return;
  let monsters = hurtEntity.dimension
    .getEntities({
      location: hurtEntity.location,
      families: ["monster"],
      maxDistance: 25,
    })
    .filter((v) => v !== hurtEntity);
  if (getSetting("abilities_target_players")?.valueOf() === true)
    monsters.push(
      ...hurtEntity.dimension.getPlayers({
        location: hurtEntity.location,
        excludeGamemodes: [GameMode.creative, GameMode.spectator],
        excludeNames: [damageSource.damagingEntity.name],
        maxDistance: 25,
      })
    );
  if (monsters.length < 1) return;
  let lastDamage = damage;
  let lastEntity = hurtEntity;
  let i = 0;
  const damageNext = () => {
    system.runTimeout(() => {
      i++;
      monsters.sort(
        (a, b) =>
          getDistance(b.location, lastEntity.location) <
          getDistance(a.location, lastEntity.location)
      );
      if (getDistance(monsters[0].location, lastEntity.location) > 5) return;

      //VISUAL
      if (getSetting("particles")?.valueOf() === true) {
        for (let i1 = 0.1; i1 < 1; i1 += 0.1) {
          spawnParticle(
            lastEntity.dimension,
            nextLocation(
              lastEntity.location,
              getDirection(
                lastEntity.location,
                addCoords(monsters[0].location, { x: 0, y: 1, z: 0 })
              ),
              i1
            ),
            "pod_rpg:chainstrike"
          );
        }
      }
      lastEntity = monsters.shift();
      world.playSound("dig.chain", lastEntity.location);
      lastDamage *= (100 - getDamageDecrease(level)) / 100;
      lastEntity.applyDamage(lastDamage, {
        cause: EntityDamageCause.magic,
        damagingEntity: damageSource.damagingEntity,
        damagingProjectile: damageSource.damagingProjectile,
      });
      if (monsters.length > 0 && i < 4) damageNext();
    }, 5);
  };
  damageNext();
};

const nextLocation = (vec, dir, length) => {
  return {
    x: vec.x + length * dir.x,
    y: vec.y + length * dir.y,
    z: vec.z + length * dir.z,
  };
};

const getCooldown = (level) => {
  let cooldown = 60;
  if (level >= 1) cooldown -= 5;
  if (level >= 4) cooldown -= 10;
  return cooldown;
};

const getDamageDecrease = (level) => {
  let value = 30;
  if (level >= 2) value -= 5;
  if (level >= 5) value -= 10;
  return value;
};

const getDuration = (level) => {
  let duration = 10;
  if (level >= 3) duration += 5;
  return duration;
};
