import { EntityHealthComponent } from "@minecraft/server";
import { rewardKill } from "../skills/fighting";
import { spawnParticle } from "../../utils/utils";

export const horde_crush_ability = {
  key: "horde_crush",
  unlock_level: 30,
  enableTitle: false,
  description: (level) => {
    return [`${getChance(level)}%%`, `${getDamageIncrease(level)}%%`];
  },
  damageIncrease: (level) => getDamageIncrease(level),
  chance: (level) => getChance(level),
};

export const handleHordeCrush = (event, level) => {
  const { hurtEntity, damageSource, damage } = event;
  if (damageSource.cause === "projectile") return;
  const entities = hurtEntity.dimension.getEntities({
    location: hurtEntity.location,
    maxDistance: 5,
    families: ["monster"],
  });
  if (entities.length > 1) {
    let additionalDamage = 0;
    for (let i = 0; i <= entities.length; i++) {
      if (getChance(level) >= Math.random() * 100) {
        additionalDamage += (damage * getDamageIncrease(level)) / 100;
      }
    }

    spawnParticle(
      hurtEntity.dimension,
      hurtEntity.location,
      "pod_rpg:hordecrush"
    );
    damageSource.damagingEntity.playSound("mob.zombie.woodbreak", {
      volume: 0.3,
      pitch: 0.8,
    });

    const component = hurtEntity.getComponent(
      EntityHealthComponent.componentId
    );
    component.setCurrentValue(component.currentValue - additionalDamage);
    if (component.currentValue <= 0)
      rewardKill(damageSource.damagingEntity, hurtEntity);
  }
};

const getChance = (level) => {
  let chance = 50;
  if (level >= 1) chance += 10;
  if (level >= 3) chance += 10;
  if (level >= 5) chance += 10;
  return chance;
};

const getDamageIncrease = (level) => {
  let value = 5;
  if (level >= 2) value += 5;
  if (level >= 4) value += 5;
  return value;
};
