import { EntityHealthComponent, system, world } from "@minecraft/server";
import { getAbilityName } from "../../utils/addonUtils";
import TimerUtil from "../../utils/cooldowns";
import { addCoords, spawnParticle } from "../../utils/utils";

export const life_drain_timer = new TimerUtil();

export const life_drain_ability = {
  key: "life_drain",
  unlock_level: 5,
  enableTitle: true,
  description: (level) => {
    return [`${getHealing(level)}`, `${getDuration(level)}`];
  },
  cooldown: (level) => getCooldown(level),
  onEnable: (data) => {
    const { player, skill, level } = data;
    life_drain_timer.set(player.name, getDuration(level) * 20);
    life_drain_timer.enableDisplay(player, () => {
      return {
        rawtext: [
          { text: "§p" },
          ...getAbilityName(life_drain_ability),
          {
            text: ` ${(
              (life_drain_timer.get(player.name) - system.currentTick) /
              20
            ).toFixed(1)}s`,
          },
        ],
      };
    });
    skill.abilityCooldown.set(player, getCooldown(level) * 20, skill);
  },
};

export const handleLifeDrain = (event, level) => {
  const { damageSource, damage } = event;
  const healthComponent = damageSource.damagingEntity.getComponent(
    EntityHealthComponent.componentId
  );
  world.playSound(
    "mob.evocation_illager.prepare_summon",
    damageSource.damagingEntity.location,
    { volume: 1, pitch: 3.75 + Math.random() / 2 }
  );
  spawnParticle(
    damageSource.damagingEntity.dimension,
    addCoords(damageSource.damagingEntity.location, { y: 1 }),
    "pod_rpg:lifedrain"
  );
  healthComponent.setCurrentValue(
    Math.min(
      healthComponent.effectiveMax,
      healthComponent.currentValue + damage * (getHealing(level) / 100)
    )
  );
};

const getCooldown = (level) => {
  let cooldown = 90;
  if (level >= 2) cooldown -= 15;
  if (level >= 5) cooldown -= 15;
  return cooldown;
};

const getHealing = (level) => {
  let value = 25;
  if (level >= 3) value += 5;
  return value;
};

const getDuration = (level) => {
  let duration = 10;
  if (level >= 1) duration += 5;
  if (level >= 4) duration += 5;
  return duration;
};
