import { GameMode, system } from "@minecraft/server";
import { arrowRegistry } from "../registry";
import { getAbilityLevel, getAbilityName } from "../../utils/addonUtils";
import { hunting_skill } from "../skills/hunting";
import {
  getDirection,
  getDistance,
  getMagnitude,
  spawnParticle,
} from "../../utils/utils";
import TimerUtil from "../../utils/cooldowns";
import { getSetting } from "../../utils/settings";

export const aim_assist_timer = new TimerUtil();

export const aim_assist_ability = {
  key: "aim_assist",
  unlock_level: 30,
  description: (level) => {
    return [`${getRadius(level)}`, `${getDuration(level)}`];
  },
  cooldown: (level) => getCooldown(level),
  enableTitle: true,
  onEnable: (data) => {
    const { player, skill, level } = data;

    aim_assist_timer.set(player.name, getDuration(level) * 20);
    aim_assist_timer.enableDisplay(player, () => {
      return {
        rawtext: [
          { text: "§p" },
          ...getAbilityName(aim_assist_ability),
          {
            text: ` ${(
              (aim_assist_timer.get(player.name) - system.currentTick) /
              20
            ).toFixed(1)}s`,
          },
        ],
      };
    });
    skill.abilityCooldown.set(player, getCooldown(level) * 20, skill);
  },
};

export const handleAimAssist = (event) => {
  const { arrow, player } = event;
  const level = getAbilityLevel(player, hunting_skill, aim_assist_ability);
  const runId = system.runInterval(() => {
    if (!arrow.isValid()) return;
    const mobs = arrow.dimension.getEntities({
      location: arrow.location,
      families: ["mob"],
      maxDistance: getRadius(level),
    });
    if (getSetting("abilities_target_players")?.valueOf() === true)
      mobs.push(
        ...arrow.dimension.getPlayers({
          location: arrow.location,
          excludeGamemodes: [GameMode.creative, GameMode.spectator],
          excludeNames: [player.name],
          maxDistance: getRadius(level),
        })
      );
    if (mobs.length === 0) return;
    mobs.sort((v) => getDistance(v.location, arrow.location)).reverse();
    const direction = getDirection(arrow.location, mobs[0].getHeadLocation());

    let speed = getMagnitude(arrow.getVelocity()) / getMagnitude(direction);
    speed = Math.min(0.25, speed);

    arrow.clearVelocity();
    arrow.applyImpulse({
      x: direction.x * speed,
      y: direction.y * speed,
      z: direction.z * speed,
    });
    spawnParticle(arrow.dimension, arrow.location, "pod_rpg:aimassist");
  });

  arrowRegistry[event.arrow.id] = {
    arrow: arrow,
    source: player,
    onImpact: () => {
      system.clearRun(runId);
    },
  };
};

const getCooldown = (level) => {
  let value = 90;
  if (level >= 2) value -= 15;
  if (level >= 4) value -= 15;
  return value;
};

const getDuration = (level) => {
  let value = 10;
  if (level >= 1) value += 5;
  if (level >= 3) value += 5;
  return value;
};

const getRadius = (level) => {
  let value = 5;
  if (level >= 5) value += 5;
  return value;
};
