import { system, world } from "@minecraft/server";
import {
  displayActionBar,
  getDirection,
  getDistance,
  getMagnitude,
} from "../../utils/utils";
import { animals } from "../skills/hunting";

export const animal_radar_ability = {
  key: "animal_radar",
  unlock_level: 15,
  enableTitle: true,
  description: () => {
    return [];
  },
  cooldown: (level) => getCooldown(level),
  onEnable: async (data) => {
    const { player, skill, level } = data;
    const entities = player.dimension.getEntities({
      location: player.location,
      maxDistance: 100,
    });
    const promises = [];
    const spawnedDisplays = [];
    for (const entity of entities) {
      promises.push(
        (async () => {
          const animalData = animals.find((v) => v.id === entity.typeId);
          if (!animalData || animalData.icon_id === undefined) return;
          const d = getDirection(player.location, entity.location);
          const m = getMagnitude(d);
          const displayLocation = {
            x: player.location.x + d.x * (2.5 / m),
            y: player.getHeadLocation().y,
            z: player.location.z + d.z * (2.5 / m),
          };
          //Merging multiple animals in one direction
          const foundDisplay = spawnedDisplays.find(
            (v) =>
              getDistance(v.location, displayLocation) <= 0.25 &&
              v.type === animalData.id
          );
          if (foundDisplay !== undefined) {
            foundDisplay.amount += 1;
            foundDisplay.displayEntity.nameTag =
              foundDisplay.displayEntity.nameTag.split("\n")[0] +
              "\n§7+" +
              foundDisplay.amount;
          } else {
            const display = player.dimension.spawnEntity(
              "pod_rpg:animal_radar",
              displayLocation
            );
            //Register spawned display
            spawnedDisplays.push({
              location: displayLocation,
              type: animalData.id,
              displayEntity: display,
              amount: 0,
            });

            //Setting icon for texture
            display.setProperty("pod_rpg:icon_id", animalData.icon_id);
            //Adding name
            display.nameTag =
              getDistance(player.location, entity.location).toFixed(0) + "m";
          }
        })()
      );
    }

    await Promise.all(promises);

    if (spawnedDisplays.length === 0) {
      displayActionBar(player, 0, {
        translate: "pod_rpg.abilities.animal_radar.no_animals",
      });
      return;
    }

    world.playSound("beacon.power", player.location);

    skill.abilityCooldown.set(player, getCooldown(level) * 20, skill);
    //Removing Entities
    system.runTimeout(async () => {
      for (const display of spawnedDisplays) {
        if (display.displayEntity && display.displayEntity.isValid())
          display.displayEntity.remove();
      }
    }, 20 * 60);
  },
};

const getCooldown = (level) => {
  return 100 - level * 10;
};
