import { world } from "@minecraft/server";
import { getSetting } from "../../utils/settings";
import {
  addCoords,
  displayActionBar,
  getDirection,
  getDistance,
  indexOfPermutationMatched,
  spawnParticle,
} from "../../utils/utils";

export const ore_radar_ability = {
  key: "ore_radar",
  unlock_level: 15,
  enableTitle: true,
  description: () => {
    return [];
  },
  cooldown: (level) => getCooldown(level),
  onEnable: async (data) => {
    const { player, skill, level } = data;

    skill.abilityCooldown.set(player, getCooldown(level) * 20, skill);
    const traced_blocks = [
      "coal_ore",
      "deepslate_coal_ore",
      "iron_ore",
      "deepslate_iron_ore",
      "redstone_ore",
      "deepslate_redstone_ore",
      "lit_redstone_ore",
      "lit_deepslate_redstone_ore",
      "lapis_ore",
      "deepslate_lapis_ore",
      "gold_ore",
      "deepslate_gold_ore",
      "emerald_ore",
      "deepslate_emerald_ore",
      "copper_ore",
      "deepslate_copper_ore",
      "quartz_ore",
      "nether_gold_ore",
    ];
    if (level >= 2) traced_blocks.push("diamond_ore", "deepslate_diamond_ore");
    if (level >= 4) traced_blocks.push("ancient_debris");
    const blocks = [];
    const playerLocationFloored = {
      x: Math.floor(player.location.x),
      y: Math.floor(player.location.y),
      z: Math.floor(player.location.z),
    };
    let highestMatch = -1;

    const promises = [];

    for (let x = -4; x < 5; x++) {
      for (let y = -4; y < 5; y++) {
        for (let z = -4; z < 5; z++) {
          const location = addCoords(playerLocationFloored, {
            x: x,
            y: y,
            z: z,
          });
          promises.push(
            (async () => {
              const tracedIndex = indexOfPermutationMatched(
                player.dimension.getBlock(location).permutation,
                traced_blocks
              );
              if (tracedIndex !== -1) {
                blocks.push({ type: tracedIndex, location: location });
                if (highestMatch < tracedIndex) highestMatch = tracedIndex;
              }
            })()
          );
        }
      }
    }

    await Promise.all(promises);

    if (blocks.length === 0) {
      displayActionBar(player, 0, {
        translate: "pod_rpg.abilities.ore_radar.no_ore_found",
      });
      return;
    }
    const closest = blocks
      .filter((v) => v.type === highestMatch)
      .sort((v) => getDistance(player.location, v.location))
      .reverse();
    player.teleport(player.location, {
      facingLocation: addCoords(closest[0].location, {
        x: 0.5,
        y: -1,
        z: 0.5,
      }),
    });

    world.playSound("beacon.power", player.location);
    //VISUAL
    if (getSetting("particles")?.valueOf() === true) {
      for (let i = 0; i < 1.05; i += 0.05) {
        spawnParticle(
          player.dimension,
          nextLocation(
            addCoords(player.location, { x: 0, y: 1.4, z: 0 }),
            getDirection(
              addCoords(player.location, { x: 0, y: 1.4, z: 0 }),
              addCoords(closest[0].location, { x: 0.5, y: 0.5, z: 0.5 })
            ),
            i
          ),
          "pod_rpg:oreradar"
        );
      }
    }

    displayActionBar(player, 1, {
      rawtext: [
        { text: "§a" },
        { translate: "pod_rpg.abilities.ore_radar.traced" },
        { text: " " },
        {
          translate: `tile.${traced_blocks[closest[0].type].replace(
            "lit_",
            ""
          )}.name`,
        },
      ],
    });
  },
};

const nextLocation = (vec, dir, length) => {
  return {
    x: vec.x + length * dir.x,
    y: vec.y + length * dir.y,
    z: vec.z + length * dir.z,
  };
};

const getCooldown = (level) => {
  let cooldown = 30;
  if (level >= 1) cooldown -= 5;
  if (level >= 3) cooldown -= 5;
  if (level >= 5) cooldown -= 10;
  return cooldown;
};
