import { system, world } from "@minecraft/server";
import { addCoords } from "../../utils/utils";
import { getSetting } from "../../utils/settings";

export const fire_rain_ability = {
  key: "fire_rain",
  unlock_level: 15,
  enableTitle: true,
  description: (level) => {
    return [`${getDuration(level)}`];
  },
  cooldown: (level) => getCooldown(level),
  onEnable: (data) => {
    const { player, skill, level } = data;

    let ticks = 0;
    const runId = system.runInterval(() => {
      ticks++;
      if (ticks >= getDuration(level) * 20) {
        system.clearRun(runId);
        return;
      }
      for (const entity of player.dimension.getEntities({
        location: player.location,
        maxDistance: 1,
        type: "pod_rpg:fireball",
      }))
        entity.remove();
      const fireball = player.dimension.spawnEntity(
        "pod_rpg:fireball",
        addCoords(player.location, {
          x: -10 + Math.random() * 20,
          y: 15 + Math.random() * 10,
          z: -10 + Math.random() * 20,
        })
      );
      world.playSound("mob.ghast.fireball", fireball.location);
      if (getSetting("fire_damage")?.valueOf() === true)
        fireball.triggerEvent("pod_rpg:fire_damage");
    });
    skill.abilityCooldown.set(player, getCooldown(level) * 20, skill);
  },
};

const getCooldown = (level) => {
  let cooldown = 120;
  if (level >= 1) cooldown -= 15;
  if (level >= 3) cooldown -= 15;
  if (level >= 5) cooldown -= 15;
  return cooldown;
};

const getDuration = (level) => {
  let value = 5;
  if (level >= 2) value += 5;
  if (level >= 4) value += 5;
  return value;
};
