import { system, world } from "@minecraft/server";
import { spawnParticle } from "../../utils/utils";

export const flame_burst_ability = {
  key: "flame_burst",
  unlock_level: 5,
  enableTitle: true,
  description: (level) => {
    return [`${getDuration(level)}`];
  },
  cooldown: (level) => getCooldown(level),
  onEnable: async (data) => {
    const { player, skill, level } = data;

    const promises = [];
    const projectiles = [];
    for (let i = -180; i < 180; i += 15) {
      promises.push(
        (async () => {
          const projectile = player.dimension.spawnEntity(
            "pod_rpg:fire_projectile",
            player.location
          );

          projectiles.push({
            projectile: projectile,
            rotation: { x: 0, y: i },
          });
          projectile.setRotation({ x: 0, y: i });
          projectile.runCommand("tp ^ ^ ^0.5");
        })()
      );
    }

    await Promise.all(promises);
    let ticks = 0;
    const runId = system.runInterval(() => {
      ticks += 2;
      if (ticks >= getDuration(level) * 20) {
        system.clearRun(runId);
        for (const projectile of projectiles) projectile.projectile.remove();
        return;
      }

      const promises2 = [];
      for (const projectile of projectiles) {
        promises2.push(
          (async () => {
            if (ticks === 2)
              projectile.projectile.triggerEvent("pod_rpg:activate");
            projectile.projectile.setRotation(projectile.rotation);
            projectile.projectile.runCommand("tp ^ ^ ^0.25");
            spawnParticle(
              projectile.projectile.dimension,
              projectile.projectile.location,
              "minecraft:basic_flame_particle"
            );
          })()
        );

        Promise.all(promises2);
      }
    }, 2);

    world.playSound("mob.ghast.fireball", player.location);
    spawnParticle(player.dimension, player.location, "pod_rpg:flameburst");
    skill.abilityCooldown.set(player, getCooldown(level) * 20, skill);
  },
};

const getCooldown = (level) => {
  let cooldown = 60;
  if (level >= 1) cooldown -= 5;
  if (level >= 3) cooldown -= 5;
  if (level >= 5) cooldown -= 5;
  return cooldown;
};

const getDuration = (level) => {
  let value = 2;
  if (level >= 2) value += 2;
  if (level >= 4) value += 2;
  return value;
};
