import { BlockPermutation, system } from "@minecraft/server";
import { addCoords, blockMatches } from "../../utils/utils";
import { getAbilityLevel } from "../../utils/addonUtils";
import { pyromaniac_skill } from "../skills/pyromaniac";

export const magma_walker_ability = {
  key: "magma_walker",
  unlock_level: 30,
  enableTitle: false,
  description: (level) => {
    return [`${getDuration(level)}%%`];
  },
};

export const handleMagmaWalker = async (player) => {
  const level = getAbilityLevel(player, pyromaniac_skill, magma_walker_ability);
  const playerBlock = player.dimension.getBlock(player.location);
  if (!playerBlock) return;

  if (blockMatches(playerBlock, ["lava", "flowing_lava", "magma"])) {
    player.teleport(addCoords(player.location, { x: 0, y: 1, z: 0 }));
    return;
  }
  const blocks = [playerBlock.below()];
  blocks.push(
    blocks[0].north(),
    blocks[0].east(),
    blocks[0].south(),
    blocks[0].west()
  );
  if (level >= 3) {
    blocks.push(
      blocks[1].east(),
      blocks[2].south(),
      blocks[3].west(),
      blocks[4].north()
    );
  }
  for (const block of blocks) {
    if (!block.isLiquid) continue;

    if (blockMatches(block, ["lava", "flowing_lava"])) {
      if (level >= 5 && !player.hasTag("pod_rpg.fireResistance")) {
        player.addEffect("fire_resistance", 20000000, { showParticles: false });
        player.addTag("pod_rpg.fireResistance");
      }
      block.setPermutation(BlockPermutation.resolve("magma"));
      system.runTimeout(async () => {
        if (
          !blockMatches(block, ["magma"]) ||
          block.dimension.getPlayers({
            location: block.location,
            maxDistance: 1,
          }).length !== 0
        ) {
          return;
        }
        block.setPermutation(BlockPermutation.resolve("lava"));
      }, getDuration(level) * 20);
    }
  }
};

const getDuration = (level) => {
  let chance = 2;
  if (level >= 1) chance += 1;
  if (level >= 2) chance += 1;
  if (level >= 4) chance += 1;
  return chance;
};
