import {
  EntityDamageCause,
  EntityHealthComponent,
  system,
  world,
} from "@minecraft/server";
import { abilityActionBar, addExp, getLevel } from "../../utils/addonUtils";
import { AbilityTimerUtil } from "../../utils/cooldowns";
import { blockMatches, isHolding } from "../../utils/utils";
import { kaboom_ability } from "../abilities/blastmaster.kaboom";
import { blast_shield_ability } from "../abilities/blastmaster.blast_shield";
import { tnt_boost_ability } from "../abilities/blastmaster.tnt_boost";
import { triggerAbilityEvent } from "../../utils/events";

export const blastmaster_skill = {
  key: "blastmaster",
  primary_color: "§m",
  secondary_color: "§c",
  abilityCooldown: new AbilityTimerUtil(),
  base_exp: 30,
  base_passive_chance: 1,
  entityHurt: {
    onEvent: async (event) => handleEntityHurt(event),
  },
  playerSneak: {
    onEvent: (event, doubleClick) => {
      if (
        !event.isSneaking ||
        !(
          isHolding(event.player, "minecraft:gunpowder") ||
          isHolding(event.player, "minecraft:tnt")
        )
      )
        return;
      abilityActionBar(event.player, blastmaster_skill, doubleClick, "Sneak");
      event.player.playSound("random.click");
      if (doubleClick) {
        triggerAbilityEvent.call({
          player: event.player,
          skill: blastmaster_skill,
          eventData: event,
        });
      }
    },
  },
  entityDie: {
    onEvent: async (event) => {
      if (
        event.damageSource.damagingEntity?.typeId === "minecraft:player" &&
        event.deadEntity.matches({ families: ["creeper"] })
      ) {
        addExp(event.damageSource.damagingEntity, blastmaster_skill, 5);
      }
    },
  },
  itemUseOn: {
    before: {
      onEvent: async (event) => {
        if (
          blockMatches(event.block, ["tnt"]) &&
          event.itemStack.typeId === "minecraft:flint_and_steel"
        ) {
          system.runTimeout(() => addExp(event.source, blastmaster_skill, 10));
        }
      },
    },
  },
  itemUse: {
    item: {
      typeIds: ["minecraft:gunpowder"],
    },
    double_click: {
      useCommon: true,
    },
  },
  abilities: [kaboom_ability, blast_shield_ability, tnt_boost_ability],
};

const handleEntityHurt = async (event) => {
  const { hurtEntity, damageSource, damage } = event;
  if (hurtEntity.typeId === "minecraft:player") {
    if (
      damageSource.cause === EntityDamageCause.blockExplosion ||
      damageSource.cause === EntityDamageCause.entityExplosion
    ) {
      addExp(hurtEntity, blastmaster_skill, Math.round(damage));
      const level = getLevel(hurtEntity, blastmaster_skill);
      if (
        level * blastmaster_skill.base_passive_chance >=
        Math.random() * 100
      ) {
        const healthComponent = hurtEntity.getComponent(
          EntityHealthComponent.componentId
        );
        if (healthComponent.currentValue > 0) {
          healthComponent.setCurrentValue(
            healthComponent.currentValue + damage
          );
          world.playSound("block.grindstone.use", hurtEntity.location);
          //PLAY PARTICLE/SOUND
        }
      }
    }
  }
};
