import { EquipmentSlot, system, world } from "@minecraft/server";
import { abilityActionBar, addExp, getLevel } from "../../utils/addonUtils";
import { AbilityTimerUtil } from "../../utils/cooldowns";
import { getSetting } from "../../utils/settings";
import { getDistance, getItemInSlot, isHolding } from "../../utils/utils";
import { dimensions } from "../../main";
import { swift_step_ability } from "../abilities/explorer.swift_step";
import { mount_ability } from "../abilities/explorer.mount";
import { homecoming_ability } from "../abilities/explorer.homecoming";
import { triggerAbilityEvent } from "../../utils/events";

export const explorer_skill = {
  key: "explorer",
  primary_color: "§s",
  secondary_color: "§b",
  abilityCooldown: new AbilityTimerUtil(),
  base_exp: 50,
  base_passive_chance: 0.01,
  tick: {
    interval: 20,
    onTick: async () => handleTick(),
  },
  playerSneak: {
    onEvent: (event, doubleClick) => {
      if (
        !event.isSneaking ||
        !(
          isHolding(event.player, "minecraft:saddle") ||
          getItemInSlot(event.player, EquipmentSlot.Mainhand)?.hasTag(
            "minecraft:boat"
          )
        )
      )
        return;
      abilityActionBar(event.player, explorer_skill, doubleClick, "Sneak");
      event.player.playSound("random.click");
      if (doubleClick) {
        triggerAbilityEvent.call({
          player: event.player,
          skill: explorer_skill,
          eventData: event,
        });
      }
    },
  },
  playerJump: {
    onEvent: async (event) => handlePlayerJump(event),
  },
  itemUse: {
    item: {
      typeIds: ["minecraft:saddle"],
      tags: ["minecraft:boat"],
    },
    double_click: {
      useCommon: true,
    },
  },
  abilities: [swift_step_ability, mount_ability, homecoming_ability],
};

const lastPos = {};
let ticks = 0;
const handleTick = async () => {
  ticks++;
  const allPlayers = world.getAllPlayers();
  for (const dimension of dimensions) {
    if (dimension.getPlayers().length !== 0) {
      for (const entity of dimension.getEntities({
        type: "pod_rpg:mount",
      })) {
        if (
          dimension.getPlayers({ location: entity.location, maxDistance: 5 })
            .length === 0
        )
          entity.triggerEvent("pod_rpg:despawn");
      }
    }
  }

  const interval = getSetting("reduce_validations")?.valueOf() === true ? 5 : 1;
  if (ticks % interval === 0) {
    for (const player of allPlayers) {
      if (lastPos[player.name]) {
        const distance = getDistance(player.location, lastPos[player.name]);
        if (distance < 15 * (interval * 2) && !player.isFlying)
          addExp(player, explorer_skill, Math.round(distance / 5), 6);
      }
      lastPos[player.name] = player.location;
    }
  }
};

let onGround = [];
const handlePlayerJump = async (event) => {
  const { player, isJumping, lastJump } = event;
  if (getSetting("double_jump")?.valueOf() === false || player.isFlying) return;
  if (lastJump !== undefined && !onGround.includes(player.name) && isJumping) {
    const sinceLastJump = system.currentTick - lastJump;
    if (
      sinceLastJump <
        (getSetting("slower_double_click")?.valueOf() === true ? 15 : 10) &&
      sinceLastJump > 1
    ) {
      const level = getLevel(player, explorer_skill);
      if (level < 5) return;
      player.applyKnockback(
        player.getViewDirection().x,
        player.getViewDirection().z,
        0.75 + 2.35 * explorer_skill.base_passive_chance * level,
        0.35 + 0.2 * explorer_skill.base_passive_chance * level
      );
      player.playSound("mob.enderdragon.flap", {
        location: player.location,
        volume: 0.5,
        pitch: 1.1,
      });
      onGround.push(player.name);
      const runId = system.runInterval(
        () => {
          if (player.isOnGround) {
            onGround = onGround.filter((v) => v !== player.name);
            system.clearRun(runId);
          }
        },
        getSetting("reduce_validations")?.valueOf() === true ? 8 : 2
      );
    }
  }
};
