import { EquipmentSlot, world } from "@minecraft/server";
import {
  abilityActionBar,
  addExp,
  getAbilityLevel,
  getLevel,
  getSelectedAbility,
} from "../../utils/addonUtils";
import {
  getItemInSlot,
  locationString,
  spawnParticle,
} from "../../utils/utils";
import { AbilityTimerUtil } from "../../utils/cooldowns";
import {
  handleLifeDrain,
  life_drain_ability,
  life_drain_timer,
} from "../abilities/fighting.life_drain";
import {
  chain_strike_ability,
  chain_strike_timer,
  handleChainStrike,
} from "../abilities/fighting.chain_strike";
import {
  handleHordeCrush,
  horde_crush_ability,
} from "../abilities/fighting.horde_crush";
import { triggerAbilityEvent } from "../../utils/events";

export const fighting_skill = {
  key: "fighting",
  primary_color: "§m",
  secondary_color: "§c",
  abilityCooldown: new AbilityTimerUtil(),
  base_exp: 40,
  base_passive_chance: 0.5,
  entityHurt: {
    onEvent: async (event) => handleEntityHurt(event),
  },
  entityDie: {
    onEvent: async (event) => {
      if (event.damageSource.damagingEntity?.typeId === "minecraft:player")
        if (event.deadEntity.matches({ families: ["monster"] }))
          rewardKill(event.damageSource.damagingEntity, event.deadEntity);
    },
  },
  playerSneak: {
    onEvent: (event, doubleClick) => {
      if (
        !event.isSneaking ||
        !getItemInSlot(event.player, EquipmentSlot.Mainhand)?.hasTag(
          "minecraft:is_sword"
        )
      )
        return;
      abilityActionBar(event.player, fighting_skill, doubleClick, "Sneak");
      event.player.playSound("random.click");
      if (doubleClick) {
        triggerAbilityEvent.call({
          player: event.player,
          skill: fighting_skill,
          eventData: event,
        });
      }
    },
  },
  itemUse: {
    item: {
      tags: ["minecraft:is_sword"],
    },
    double_click: {
      useCommon: true,
    },
  },
  abilities: [life_drain_ability, chain_strike_ability, horde_crush_ability],
};

const handleEntityHurt = async (event) => {
  const { damageSource, hurtEntity } = event;
  if (!hurtEntity?.isValid()) return;
  const isPlayer = hurtEntity.matches({ type: "minecraft:player" });
  if (
    (hurtEntity.matches({ families: ["monster"] }) || isPlayer) &&
    damageSource.damagingEntity?.typeId === "minecraft:player"
  ) {
    const selectedAbility = getSelectedAbility(
      damageSource.damagingEntity,
      fighting_skill
    );

    if (!selectedAbility) return;

    const abilityLevel = getAbilityLevel(
      damageSource.damagingEntity,
      fighting_skill,
      selectedAbility
    );

    //HORDE CRUSH
    if (selectedAbility && !isPlayer && selectedAbility.key === "horde_crush") {
      handleHordeCrush(event, abilityLevel);
    }

    //LIFE DRAIN
    if (life_drain_timer.has(damageSource.damagingEntity.name)) {
      handleLifeDrain(event, abilityLevel);
    }

    //CHAIN STRIKE
    if (!isPlayer && chain_strike_timer.has(damageSource.damagingEntity.name)) {
      handleChainStrike(event, abilityLevel);
    }
  }
};

export const rewardKill = (player, entity) => {
  addExp(player, fighting_skill, 5);
  if (
    fighting_skill.base_passive_chance * getLevel(player, fighting_skill) >=
    Math.random() * 100
  ) {
    if (
      player.runCommand(
        `loot spawn ${locationString(entity.location)} loot "entities/${
          entity.typeId.split(":")[1]
        }" mainhand`
      ).successCount !== 0
    ) {
      spawnParticle(player.dimension, entity.location, "pod_rpg:extra_drop");
      world.playSound("random.pop", entity.location);
    }
  }
};
