import { abilityActionBar, addExp, getLevel } from "../../utils/addonUtils";
import { isHolding, locationString, spawnParticle } from "../../utils/utils";
import { AbilityTimerUtil } from "../../utils/cooldowns";
import { saturation_ability } from "../abilities/hunting.saturation";
import {
  aim_assist_ability,
  aim_assist_timer,
  handleAimAssist,
} from "../abilities/hunting.aim_assist";
import { animal_radar_ability } from "../abilities/hunting.animal_radar";
import { world } from "@minecraft/server";
import { triggerAbilityEvent } from "../../utils/events";
import { pyromaniac_skill } from "./pyromaniac";

export const hunting_skill = {
  key: "hunting",
  primary_color: "§2",
  secondary_color: "§q",
  abilityCooldown: new AbilityTimerUtil(),
  base_exp: 30,
  base_passive_chance: 1,
  playerShootArrow: {
    onEvent: async (event) => handleShootArrow(event),
  },
  entityDie: {
    onEvent: async (event) => {
      if (event.damageSource.damagingEntity?.typeId === "minecraft:player") {
        const animalData = animals.find(
          (v) => v.id === event.deadEntity.typeId
        );
        if (animalData)
          rewardKill(
            event.damageSource.damagingEntity,
            event.deadEntity,
            animalData
          );
      }
    },
  },
  playerSneak: {
    onEvent: (event, doubleClick) => {
      if (
        !event.isSneaking ||
        !(
          isHolding(event.player, "minecraft:shears") ||
          isHolding(event.player, "minecraft:leather") ||
          isHolding(event.player, "minecraft:feather")
        )
      )
        return;
      abilityActionBar(event.player, hunting_skill, doubleClick, "Sneak");
      event.player.playSound("random.click");
      if (doubleClick) {
        triggerAbilityEvent.call({
          player: event.player,
          skill: pyromaniac_skill,
          eventData: event,
        });
      }
    },
  },
  itemUse: {
    item: {
      typeIds: ["minecraft:shears", "minecraft:feather", "minecraft:leather"],
    },
    double_click: {
      useCommon: true,
    },
  },
  abilities: [saturation_ability, animal_radar_ability, aim_assist_ability],
};

const handleShootArrow = async (event) => {
  const { player } = event;
  if (aim_assist_timer.has(player.name)) handleAimAssist(event);
};

export const rewardKill = (player, entity, animalData) => {
  addExp(player, hunting_skill, animalData.exp);
  if (
    hunting_skill.base_passive_chance * getLevel(player, hunting_skill) >=
    Math.random() * 100
  ) {
    if (
      player.runCommand(
        `loot spawn ${locationString(entity.location)} loot "${
          animalData.loot
            ? animalData.loot
            : `entities/${entity.typeId.split(":")[1]}`
        }" mainhand`
      ).successCount !== 0
    ) {
      spawnParticle(player.dimension, entity.location, "pod_rpg:extra_drop");
      world.playSound("random.pop", entity.location);
    }
  }
};

export const animals = [
  {
    id: "minecraft:chicken",
    icon_id: 1,
    exp: 3,
  },
  {
    id: "minecraft:cow",
    icon_id: 2,
    exp: 7,
  },
  {
    id: "minecraft:sheep",
    icon_id: 3,
    exp: 6,
  },
  {
    id: "minecraft:pig",
    icon_id: 4,
    exp: 5,
  },
  {
    id: "minecraft:cod",
    loot: "entities/fish",
    exp: 3,
  },
  {
    id: "minecraft:salmon",
    loot: "entities/salmon_normal",
    exp: 3,
  },
  {
    id: "minecraft:pufferfish",
    icon_id: 5,
    exp: 5,
  },
  {
    id: "minecraft:tropicalfish",
    icon_id: 6,
    exp: 4,
  },
  {
    id: "minecraft:frog",
    icon_id: 7,
    exp: 5,
  },
  {
    id: "minecraft:squid",
    icon_id: 8,
    exp: 5,
  },
  {
    id: "minecraft:glow_squid",
    icon_id: 9,
    exp: 7,
  },
  {
    id: "minecraft:mooshroom",
    icon_id: 10,
    exp: 10,
  },
  {
    id: "minecraft:rabbit",
    icon_id: 11,
    exp: 5,
  },
];
