import {
  abilityActionBar,
  addExp,
  dropAdditionalItem,
  getLevel,
} from "../../utils/addonUtils";
import {
  addCoords,
  getItemInSlot,
  permutationMatches,
} from "../../utils/utils";
import { AbilityTimerUtil } from "../../utils/cooldowns";
import { ore_radar_ability } from "../abilities/mining.ore_radar";
import { haste_ability } from "../abilities/mining.haste";

import {
  super_break_ability,
  super_break_timer,
} from "../abilities/mining.super_break";
import { EquipmentSlot } from "@minecraft/server";
import { triggerAbilityEvent } from "../../utils/events";

export const mining_skill = {
  key: "mining",
  primary_color: "§e",
  secondary_color: "§6",
  abilityCooldown: new AbilityTimerUtil(),
  base_exp: 50,
  base_passive_chance: 0.5,
  playerBreakBlock: {
    disallowPlayerPlacedBlocks: true,
    item: {
      //typeIds: ["minecraft:diamond_pickaxe"],
      tags: ["minecraft:is_pickaxe"],
    },
    onEvent: (event) => rewardBlockBreak(event),
  },
  playerSneak: {
    onEvent: (event, doubleClick) => {
      if (
        !event.isSneaking ||
        !getItemInSlot(event.player, EquipmentSlot.Mainhand)?.hasTag(
          "minecraft:is_pickaxe"
        )
      )
        return;
      abilityActionBar(event.player, mining_skill, doubleClick, "Sneak");
      event.player.playSound("random.click");
      if (doubleClick) {
        triggerAbilityEvent.call({
          player: event.player,
          skill: mining_skill,
          eventData: event,
        });
      }
    },
  },
  itemUse: {
    item: {
      //typeIds: ["minecraft:diamond_pickaxe"],
      tags: ["minecraft:is_pickaxe"],
    },
    double_click: {
      useCommon: true,
    },
  },
  abilities: [haste_ability, ore_radar_ability, super_break_ability],
};

const rewardBlockBreak = (event) => {
  const { brokenBlockPermutation, player, block } = event;

  for (const reward of mining_rewards) {
    if (permutationMatches(brokenBlockPermutation, reward.blocks)) {
      //EXP REWARD
      addExp(player, mining_skill, reward.exp);

      //SUPER BREAK ABILITY
      if (super_break_timer.has(player.name)) {
        const skip =
          (player.dimension.id === "minecraft:overworld" &&
            block.location.y < -58) ||
          (player.dimension.id === "minecraft:nether" &&
            (121 < block.location.y || block.location.y < 6)) ||
          player.dimension.id === "minecraft:the_end";
        if (!skip)
          player.runCommandAsync(
            `execute positioned ${block.location.x} ${block.location.y} ${block.location.z} run fill ~-1 ~-1 ~-1 ~1 ~1 ~1 air destroy`
          );
      }

      //ITEM DROP
      if (!reward.item) break;
      const chance =
        getLevel(player, mining_skill) * mining_skill.base_passive_chance;
      dropAdditionalItem(
        addCoords(block.location, { x: 0.5, y: 0.5, z: 0.5 }),
        player.dimension,
        chance,
        reward.item,
        reward.item_count || 1
      );
      break;
    }
  }
};

export const mining_rewards = [
  {
    blocks: [
      "minecraft:stone",
      "minecraft:deepslate",
      "minecraft:tuff",
      "minecraft:blackstone",
      "minecraft:basalt",
      "minecraft:polished_basalt",
      "minecraft:netherrack",
      "minecraft:dripstone_block",
    ],
    exp: 1,
  },
  {
    blocks: ["minecraft:coal_ore", "deepslate_coal_ore"],
    item: "minecraft:coal",
    exp: 3,
  },
  {
    blocks: ["minecraft:quartz_ore"],
    item: "minecraft:quartz",
    item_count: 2,
    exp: 3,
  },
  {
    blocks: ["minecraft:nether_gold_ore"],
    item: "minecraft:gold_nugget",
    item_count: 4,
    exp: 3,
  },
  {
    blocks: ["minecraft:sculk"],
    exp: 3,
  },
  {
    blocks: ["minecraft:iron_ore", "minecraft:deepslate_iron_ore"],
    item: "minecraft:raw_iron",
    exp: 5,
  },
  {
    blocks: ["minecraft:copper_ore", "minecraft:deepslate_copper_ore"],
    item: "minecraft:raw_copper",
    exp: 5,
  },
  {
    blocks: [
      "minecraft:redstone_ore",
      "minecraft:deepslate_redstone_ore",
      "minecraft:lit_redstone_ore",
      "minecraft:lit_deepslate_redstone_ore",
    ],
    item: "minecraft:redstone",
    item_count: 3,
    exp: 7,
  },
  {
    blocks: ["minecraft:lapis_ore", "minecraft:deepslate_lapis_ore"],
    item: "minecraft:lapis_lazuli",
    item_count: 5,
    exp: 7,
  },
  {
    blocks: ["minecraft:gold_ore", "minecraft:deepslate_gold_ore"],
    item: "minecraft:raw_gold",
    exp: 10,
  },
  {
    blocks: ["minecraft:emerald_ore", "minecraft:deepslate_emerald_ore"],
    item: "minecraft:emerald",
    exp: 15,
  },
  {
    blocks: ["minecraft:diamond_ore", "minecraft:deepslate_diamond_ore"],
    item: "minecraft:diamond",
    exp: 20,
  },
  {
    blocks: ["minecraft:ancient_debris"],
    item: "minecraft:ancient_debris",
    exp: 50,
  },
];
