import {
  EntityDamageCause,
  EntityHealthComponent,
  system,
  world,
} from "@minecraft/server";
import {
  abilityActionBar,
  addExp,
  getLevel,
  getSelectedAbility,
} from "../../utils/addonUtils";
import { AbilityTimerUtil } from "../../utils/cooldowns";
import { triggerAbilityEvent } from "../../utils/events";
import { isHolding } from "../../utils/utils";
import { flame_burst_ability } from "../abilities/pyromaniac.flame_burst";
import { fire_rain_ability } from "../abilities/pyromaniac.fire_rain";
import {
  handleMagmaWalker,
  magma_walker_ability,
} from "../abilities/pyromaniac.magma_walker";

export const pyromaniac_skill = {
  key: "pyromaniac",
  primary_color: "§m",
  secondary_color: "§c",
  abilityCooldown: new AbilityTimerUtil(),
  base_exp: 30,
  base_passive_chance: 1,
  entityHurt: {
    onEvent: async (event) => handleEntityHurt(event),
  },
  entityDie: {
    onEvent: async (event) => {
      if (
        event.damageSource.damagingEntity?.typeId === "minecraft:player" &&
        event.deadEntity &&
        (event.deadEntity.matches({ families: ["blaze"] }) ||
          event.deadEntity.matches({ families: ["magma_cube"] }))
      ) {
        addExp(event.damageSource.damagingEntity, pyromaniac_skill, 7);
      }
    },
  },
  playerSneak: {
    onEvent: (event, doubleClick) => {
      if (
        !event.isSneaking ||
        !(
          isHolding(event.player, "minecraft:flint_and_steel") ||
          isHolding(event.player, "minecraft:flint")
        )
      )
        return;
      abilityActionBar(event.player, pyromaniac_skill, doubleClick, "Sneak");
      event.player.playSound("random.click");
      if (doubleClick) {
        triggerAbilityEvent.call({
          player: event.player,
          skill: pyromaniac_skill,
          eventData: event,
        });
      }
    },
  },
  tick: {
    onTick: async () => handleTick(),
  },
  itemUseOn: {
    before: {
      onEvent: async (event) => {
        if (event.itemStack.typeId === "minecraft:flint_and_steel") {
          system.runTimeout(() => addExp(event.source, pyromaniac_skill, 5));
        }
      },
    },
  },
  itemUse: {
    item: {
      typeIds: ["minecraft:flint"],
    },
    double_click: {
      useCommon: true,
    },
  },
  abilities: [flame_burst_ability, fire_rain_ability, magma_walker_ability],
};

const handleTick = async () => {
  const promises = [];
  for (const player of world.getAllPlayers()) {
    if (getSelectedAbility(player, pyromaniac_skill)?.key === "magma_walker")
      promises.push(handleMagmaWalker(player));
  }
  await Promise.all(promises);
};

const handleEntityHurt = async (event) => {
  const { hurtEntity, damageSource, damage } = event;
  if (hurtEntity.typeId === "minecraft:player") {
    if (
      damageSource.cause === EntityDamageCause.fire ||
      damageSource.cause === EntityDamageCause.fireTick ||
      damageSource.cause === EntityDamageCause.lava
    ) {
      addExp(hurtEntity, pyromaniac_skill, Math.round(damage));
      const level = getLevel(hurtEntity, pyromaniac_skill);
      if (level * pyromaniac_skill.base_passive_chance >= Math.random() * 100) {
        const healthComponent = hurtEntity.getComponent(
          EntityHealthComponent.componentId
        );
        if (healthComponent.currentValue > 0) {
          healthComponent.setCurrentValue(
            healthComponent.currentValue + damage
          );
          world.playSound("block.grindstone.use", hurtEntity.location);
        }
      }
    }
  }
};
