import {
  abilityActionBar,
  addExp,
  dropAdditionalItem,
  getAbilityLevel,
  getLevel,
  getSelectedAbility,
} from "../../utils/addonUtils";
import {
  addCoords,
  getItemInSlot,
  permutationMatches,
} from "../../utils/utils";
import { AbilityTimerUtil } from "../../utils/cooldowns";
import { tree_exp_ability } from "../abilities/tree_cutting.tree_exp";
import {
  fell_tree_ability,
  fell_tree_timer,
  handleFellTree,
} from "../abilities/tree_cutting.fell_tree";
import {
  handleRegrowth,
  regrowth_ability,
} from "../abilities/tree_cutting.regrowth";
import { EquipmentSlot } from "@minecraft/server";
import { triggerAbilityEvent } from "../../utils/events";

export const tree_cutting_skill = {
  key: "tree_cutting",
  primary_color: "§2",
  secondary_color: "§q",
  abilityCooldown: new AbilityTimerUtil(),
  base_exp: 50,
  base_passive_chance: 1,
  playerBreakBlock: {
    disallowPlayerPlacedBlocks: true,
    item: {
      tags: ["minecraft:is_axe"],
    },
    onEvent: (event) => rewardBlockBreak(event),
  },
  playerSneak: {
    onEvent: (event, doubleClick) => {
      if (
        !event.isSneaking ||
        !getItemInSlot(event.player, EquipmentSlot.Mainhand)?.hasTag(
          "minecraft:is_axe"
        )
      )
        return;
      abilityActionBar(event.player, tree_cutting_skill, doubleClick, "Sneak");
      event.player.playSound("random.click");
      if (doubleClick) {
        triggerAbilityEvent.call({
          player: event.player,
          skill: tree_cutting_skill,
          eventData: event,
        });
      }
    },
  },
  itemUse: {
    item: {
      tags: ["minecraft:is_axe"],
    },
    double_click: {
      useCommon: true,
    },
  },
  abilities: [regrowth_ability, tree_exp_ability, fell_tree_ability],
};

const rewardBlockBreak = (event) => {
  const { brokenBlockPermutation, player, block } = event;

  for (const reward of tree_cutting_rewards) {
    if (permutationMatches(brokenBlockPermutation, reward.blocks)) {
      //EXP REWARD
      addExp(player, tree_cutting_skill, reward.exp);

      //TREE FELLER ABILITY
      if (fell_tree_timer.has(player.name)) handleFellTree(block);

      //EXPERIENCE ABILITY
      const selectedAbility = getSelectedAbility(player, tree_cutting_skill);
      if (selectedAbility) {
        const abilityLevel = getAbilityLevel(
          player,
          tree_cutting_skill,
          selectedAbility
        );
        switch (selectedAbility.key) {
          case "regrowth":
            handleRegrowth(block, reward, abilityLevel);
            break;
          case "tree_exp":
            if (selectedAbility.chance(abilityLevel) >= Math.random() * 100) {
              for (let i = 0; i < (abilityLevel >= 4 ? 2 : 1); i++) {
                player.dimension.spawnEntity(
                  "minecraft:xp_orb",
                  addCoords(block.location, { x: 0.5, y: 0.5, z: 0.5 })
                );
              }
            }
            break;
        }
      }

      //ITEM DROP
      if (!reward.item) break;
      const chance =
        getLevel(player, tree_cutting_skill) *
        tree_cutting_skill.base_passive_chance;
      dropAdditionalItem(
        addCoords(block.location, { x: 0.5, y: 0.5, z: 0.5 }),
        player.dimension,
        chance,
        reward.item,
        reward.item_count || 1
      );
      break;
    }
  }
};

export const tree_cutting_rewards = [
  {
    blocks: ["oak_log", "stripped_oak_log"],
    item: "minecraft:oak_log",
    sapling: { blockName: "sapling", states: { sapling_type: "oak" } },
    exp: 5,
  },
  {
    blocks: ["spruce_log", "stripped_spruce_log"],
    item: "minecraft:spruce_log",
    sapling: { blockName: "sapling", states: { sapling_type: "spruce" } },
    exp: 5,
  },
  {
    blocks: ["birch_log", "stripped_birch_log"],
    item: "minecraft:birch_log",
    sapling: { blockName: "sapling", states: { sapling_type: "birch" } },
    exp: 5,
  },
  {
    blocks: ["jungle_log", "stripped_jungle_log"],
    item: "minecraft:jungle_log",
    sapling: { blockName: "sapling", states: { sapling_type: "jungle" } },
    exp: 5,
  },
  {
    blocks: ["acacia_log", "stripped_acacia_log"],
    item: "minecraft:acacia_log",
    sapling: { blockName: "sapling", states: { sapling_type: "acacia" } },
    exp: 5,
  },
  {
    blocks: ["dark_oak_log", "stripped_dark_oak_log"],
    item: "minecraft:dark_oak_log",
    sapling: { blockName: "sapling", states: { sapling_type: "dark_oak" } },
    exp: 5,
  },
  {
    blocks: ["mangrove_log", "stripped_mangrove_log"],
    item: "minecraft:mangrove_log",
    sapling: { blockName: "mangrove_propagule", states: {} },
    exp: 5,
  },
  {
    blocks: ["mangrove_wood", "stripped_mangrove_wood"],
    item: "minecraft:mangrove_wood",
    sapling: { blockName: "mangrove_propagule", states: {} },
    exp: 5,
  },
  {
    blocks: ["cherry_log", "stripped_cherry_log"],
    item: "minecraft:cherry_log",
    sapling: { blockName: "cherry_sapling", states: {} },
    exp: 5,
  },
  {
    blocks: ["cherry_wood", "stripped_cherry_wood"],
    item: "minecraft:cherry_wood",
    sapling: { blockName: "cherry_sapling", states: {} },
    exp: 5,
  },
  {
    blocks: ["crimson_stem"],
    item: "minecraft:crimson_stem",
    exp: 5,
  },
  {
    blocks: ["warped_stem"],
    item: "minecraft:warped_stem",
    exp: 5,
  },
];
