import { system, world } from "@minecraft/server";
import { skills } from "../main";
import { getSetting } from "../utils/settings";
import { blockMatches, displayActionBar } from "../utils/utils";
import { abilityActionBar, getSkillName } from "../utils/addonUtils";
import { triggerAbilityEvent } from "../utils/events";
import { bowUseData } from "./entitySpawn.a.w";
import { showMainUi, showSkillInfoPage } from "../ui";
import TimerUtil from "../utils/cooldowns";
const clicks = {};

world.afterEvents.itemUse.subscribe((event) => {
  const { itemStack, source } = event;
  if (itemStack.typeId === "pod_rpg:guide") {
    showMainUi(event.source);
    return;
  }

  const isDoubleClick =
    clicks[source.name] &&
    system.currentTick - clicks[source.name].tick <
      (getSetting("slower_double_click")?.valueOf() === true ? 41 : 21) &&
    itemStack.typeId === clicks[source.name].item &&
    source.isSneaking === clicks[source.name].sneak;

  for (const skill of skills.filter((skill) => skill.itemUse !== undefined)) {
    if (
      (skill.itemUse.item.tags &&
        skill.itemUse.item.tags.every((tag) => itemStack.hasTag(tag))) ||
      (skill.itemUse.item.typeIds &&
        skill.itemUse.item.typeIds.includes(itemStack.typeId))
    ) {
      const doubleClick = skill.itemUse.double_click.useCommon
        ? common(skill)
        : skill.itemUse.double_click;
      const action = source.isSneaking ? doubleClick.sneak : doubleClick.normal;
      action.onEvent(event, isDoubleClick);
    }
  }

  clicks[source.name] = {
    tick: system.currentTick,
    item: itemStack.typeId,
    sneak: source.isSneaking,
  };
});

const common = (skill) => {
  return {
    sneak: {
      onEvent: (event, doubleClick) => {
        event.source.playSound("random.click");
        displayActionBar(event.source, 1, {
          rawtext: [
            ...getSkillName(skill, skill.primary_color),
            {
              text: ` §8- §aClick §7- ${doubleClick ? "§a" : "§7"}Click §8- `,
            },
            { translate: "pod_rpg.common.open_menu" },
          ],
        });
        if (doubleClick) showSkillInfoPage(event.source, skill);
      },
    },
    normal: {
      onEvent: (event, doubleClick) => {
        //Check so player doesn't accidently use ability while trying to create farmland
        if (skill.key === "farming" && clicked_grass.has(event.source.id)) {
          system.runTimeout(() => {
            if (clicks[event.source.name])
              clicks[event.source.name].item = undefined;
          });
          return;
        }

        event.source.playSound("random.click");
        abilityActionBar(event.source, skill, doubleClick, "Click");
        if (doubleClick) {
          triggerAbilityEvent.call({
            player: event.source,
            skill: skill,
            eventData: event,
          });
        }
      },
    },
  };
};

//Check so player doesn't accidently use ability while trying to create farmland
const clicked_grass = new TimerUtil();
world.beforeEvents.itemUseOn.subscribe((event) => {
  if (
    blockMatches(event.block, [
      "grass_block",
      "dirt",
      "dirt_with_roots",
      "grass_path",
    ])
  )
    clicked_grass.set(event.source.id, 5);
});

world.afterEvents.itemReleaseUse.subscribe((event) => {
  const usedFor = 72000 - event.useDuration;
  if (usedFor >= 3) {
    bowUseData[event.source.name] = {
      tick: system.currentTick,
      useDuration: usedFor,
      bowItem: event.itemStack,
    };
  }
});
