import { world } from "@minecraft/server";
import { skills } from "./main";
import {
  getAbilityLevel,
  getAbilityName,
  getSkillName,
  showResetUi,
} from "./utils/addonUtils";
import { showSettingsDisclaimer } from "./utils/settings";
import { RawtextBuilder, showPage } from "./utils/uiUtils";
import {
  getLevelFromExp,
  getExpRequiredForLevel,
  getRenderableExpBar,
  MAX_LEVEL,
  getSelectedAbility,
} from "./utils/addonUtils";
import { getLevel, setSelectedAbility } from "./utils/addonUtils";
import { triggerAbilityEvent } from "./utils/events";
import { chargeAbility } from "./data/skills/archer";

export const showMainUi = (player) => {
  const body = new RawtextBuilder();
  const buttons = [];

  body.pushLine({ translate: "pod_rpg.ui.main.body1" });
  body.pushLine();
  body.pushLine({ translate: "pod_rpg.ui.main.body2" });

  buttons.push({
    translation_key: "pod_rpg.ui.skills_title",
    icon_path: "textures/podcrash/rpg/icons/skills",
    action: () => showSkillsUi(player),
  });
  buttons.push({
    translation_key: "pod_rpg.ui.guide_title",
    icon_path: "textures/podcrash/rpg/icons/guide",
    action: () => showGuideMainUi(player),
  });
  if (world.getAllPlayers().length > 1) {
    buttons.push({
      translation_key: "pod_rpg.ui.view_players",
      icon_path: "textures/podcrash/rpg/icons/view_players",
      action: () => showViewPlayersUi(player),
    });
  }
  buttons.push({
    translation_key: "pod_rpg.settings.title",
    icon_path: "textures/podcrash/rpg/icons/settings",
    action: () => showSettingsDisclaimer(player),
  });
  buttons.push({
    translation_key: "pod_rpg.reset.title2",
    icon_path: "textures/podcrash/rpg/icons/reset",
    action: () => showResetUi(player),
  });
  buttons.push({ translation_key: "pod_rpg.common.ui.exit_button" });
  showPage(
    player,
    { translate: "pod_rpg.ui.addon_name" },
    body.build(),
    buttons
  );
};

const showViewPlayersUi = (player) => {
  const body = new RawtextBuilder();
  body.pushLine({ translate: "pod_rpg.ui.view_players.choose" });
  const buttons = [];
  for (const otherPlayer of world.getAllPlayers()) {
    buttons.push({
      translation_key: otherPlayer.name,
      action: () => showSkillsUi(player, otherPlayer),
    });
  }
  buttons.push({
    translation_key: "pod_rpg.common.ui.back_button",
    action: () => showMainUi(player),
  });

  showPage(
    player,
    { translate: "pod_rpg.ui.view_players" },
    body.build(),
    buttons
  );
};

export const showSkillsUi = (player, ofPlayer = undefined) => {
  const body = new RawtextBuilder();
  body.pushLine({ translate: "pod_rpg.ui.skills_page.body" });
  const buttons = [];

  for (const skill of skills) {
    buttons.push({
      translation_key: `pod_rpg.skills.${skill.key}.name`,
      icon_path: `textures/podcrash/rpg/icons/skills/${skill.key}`,
      action: () => {
        showSkillInfoPage(player, skill, ofPlayer);
      },
    });
  }
  buttons.push({
    translation_key: "pod_rpg.common.ui.back_button",
    action: () => (ofPlayer ? showViewPlayersUi(player) : showMainUi(player)),
  });
  showPage(
    player,
    ofPlayer
      ? { text: ofPlayer.name }
      : { translate: "pod_rpg.ui.skills_title" },
    body.build(),
    buttons
  );
};

const showGuideMainUi = (player) => {
  const body = new RawtextBuilder();
  const buttons = [];

  body.pushLine({ translate: `pod_rpg.ui.guide.body1` });
  body.pushLine();
  body.pushLine({ translate: `pod_rpg.ui.guide.body2` });
  body.pushLine();
  body.pushLine({ translate: `pod_rpg.ui.guide.body3` });
  body.pushLine();
  body.pushLine({ translate: `pod_rpg.ui.guide.body4` });
  body.pushLine();

  buttons.push({
    translation_key: "pod_rpg.ui.skills_title",
    action: () => showGuideSkillUi(player),
  });
  buttons.push({
    translation_key: "pod_rpg.ui.abilities_title",
    action: () => showGuideAbilityUi(player),
  });
  buttons.push({
    translation_key: "pod_rpg.common.ui.back_button",
    action: () => showMainUi(player),
  });
  showPage(
    player,
    { translate: "pod_rpg.ui.guide_title" },
    body.build(),
    buttons
  );
};

const showGuideSkillUi = (player) => {
  const body = new RawtextBuilder();
  const buttons = [];
  for (const skill of skills) {
    body.pushLine(...getSkillName(skill, "§l"));
    body.pushLine();
    body.pushLine({ translate: `pod_rpg.skills.${skill.key}.description` });
    body.pushLine();
  }
  buttons.push({
    translation_key: "pod_rpg.common.ui.back_button",
    action: () => showGuideMainUi(player),
  });
  showPage(
    player,
    { translate: "pod_rpg.ui.skills_title" },
    body.build(),
    buttons
  );
};

const showGuideAbilityUi = (player) => {
  const body = new RawtextBuilder();
  const buttons = [];
  for (const skill of skills) {
    for (const ability of skill.abilities) {
      body.pushLine(...getAbilityName(ability, "§l"));
      body.pushLine();
      body.pushLine({
        translate: `pod_rpg.abilities.${ability.key}.description`,
        with: ability.description(0),
      });
      body.pushLine();
    }
  }
  buttons.push({
    translation_key: "pod_rpg.common.ui.back_button",
    action: () => showGuideMainUi(player),
  });
  showPage(
    player,
    { translate: "pod_rpg.ui.abilities_title" },
    body.build(),
    buttons
  );
};

export const showSkillInfoPage = (toPlayer, skill, ofPlayer = undefined) => {
  const body = new RawtextBuilder();
  const buttons = [];
  const player = ofPlayer ? ofPlayer : toPlayer;

  const exp = player.getDynamicProperty(`pod_rpg_${skill.key}_exp`) || 0;
  const level = getLevelFromExp(skill, exp);
  const expRequiredForThislevel = getExpRequiredForLevel(skill, level);
  const levelExp = exp - expRequiredForThislevel;
  const expRequired =
    getExpRequiredForLevel(skill, level + 1) - expRequiredForThislevel;

  body.pushLine({ text: "\n§l" }, ...getSkillName(skill, skill.primary_color));
  body.pushLine(
    { text: "§7" },
    { translate: "pod_rpg.common.ui.level_text", with: [`${level}`] }
  );
  body.pushLine({
    text: `\n§7[${getRenderableExpBar(
      level === MAX_LEVEL ? 100 : levelExp,
      level === MAX_LEVEL ? 100 : expRequired,
      50
    )}§7]`,
  });
  body.pushLine(
    level === MAX_LEVEL
      ? { translate: "pod_rpg.common.ui.max_level" }
      : {
          text: `§8${((levelExp / expRequired) * 100).toFixed(
            1
          )}%% (${levelExp}/${expRequired})`,
        }
  );

  const selectedAbility = getSelectedAbility(player, skill);

  if (!ofPlayer) {
    if (
      selectedAbility &&
      !skill.abilityCooldown.has(player.id + selectedAbility.key)
    ) {
      buttons.push({
        translation_key: "pod_rpg.common.ui.enable_ability",
        action: () => {
          if (
            skill.key === "archer" &&
            selectedAbility.key !== "arrow_shield"
          ) {
            chargeAbility(player);
            return;
          }
          triggerAbilityEvent.call({
            player: player,
            skill: skill,
          });
        },
      });
    }
    body.pushLine();
    body.pushLine({ translate: `pod_rpg.skills.${skill.key}.description` });
    body.pushLine(
      { text: "\n§l" },
      { translate: "pod_rpg.common.ui.passive_title" }
    );
    body.pushLine({
      translate: `pod_rpg.skills.${skill.key}.passive.description`,
      with: [skill.base_passive_chance * level + "%%"],
    });

    body.pushLine(
      { text: "\n§l" },
      { translate: "pod_rpg.ui.abilities_title" }
    );
    body.pushLine(
      { translate: "pod_rpg.common.ui.ability.description" },
      { text: "\n" }
    );
    body.pushLine(
      { translate: `pod_rpg.skills.${skill.key}.abilities.description` },
      { text: "\n" }
    );

    for (const ability of skill.abilities) {
      buttons.push({
        translation_key: `pod_rpg.abilities.${ability.key}.name`,
        icon_path: `textures/podcrash/rpg/icons/abilities/${
          ability.unlock_level <= level ? "" : "gray/"
        }${ability.key}.png`,
        action: () => {
          showAbilityPage(player, skill, ability);
        },
      });

      const abilityText =
        ability.unlock_level <= level
          ? [
              {
                translate: "pod_rpg.common.ui.ability.unlocked",
              },
            ]
          : [
              { text: `§8[§c` },
              {
                translate: "pod_rpg.common.ui.level_text",
                with: [`${ability.unlock_level}`],
              },
              { text: `§8] §r` },
            ];
      body.pushLine(
        ...abilityText,
        ...getAbilityName(ability, skill.secondary_color)
      );
    }
  } else {
    body.pushLine(
      { text: "\n§l" },
      { translate: "pod_rpg.ui.abilities_title" }
    );
    body.pushLine();

    for (const ability of skill.abilities) {
      body.push(...getAbilityName(ability, skill.secondary_color));
      if (ability.unlock_level <= level)
        body.pushLine(
          { text: ` §7- §a` },
          {
            translate: "pod_rpg.common.ui.level_text",
            with: [`${getAbilityLevel(player, skill, ability)}`],
          }
        );
      else
        body.pushLine(
          { text: ` §7- §c` },
          {
            translate: "pod_rpg.common.ui.ability.not_unlocked_short",
          }
        );
    }
  }

  const selectedAbilityText = selectedAbility
    ? getAbilityName(selectedAbility, skill.secondary_color)
    : [{ translate: "pod_rpg.common.ui.ability.non_selected" }];
  body.pushLine(
    { text: "\n§f" },
    { translate: "pod_rpg.common.ui.ability.selected" },
    ...selectedAbilityText
  );

  buttons.push({
    translation_key: "pod_rpg.common.ui.back_button",
    action: () => showSkillsUi(toPlayer, ofPlayer),
  });

  showPage(
    toPlayer,
    {
      rawtext: [
        ofPlayer
          ? { text: ofPlayer.name }
          : { translate: `pod_rpg.skills.${skill.key}.name` },
      ],
    },
    body.build(),
    buttons
  );
};

const showAbilityPage = (player, skill, ability) => {
  const body = new RawtextBuilder();
  const buttons = [];
  const level = getLevel(player, skill);
  const abilityLevel =
    player.getDynamicProperty(`pod_rpg_${skill.key}_${ability.key}`) || 0;

  body.pushLine(
    { text: "\n§l" },
    ...getAbilityName(ability, skill.secondary_color)
  );
  body.pushLine(
    { text: "§7" },
    { translate: "pod_rpg.common.ui.level_text", with: [`${abilityLevel}`] }
  );
  body.pushLine();

  body.pushLine({
    translate: `pod_rpg.abilities.${ability.key}.description`,
    with: ability.description(abilityLevel),
  });
  if (ability.cooldown) {
    body.pushLine({
      translate: "pod_rpg.common.ui.ability.cooldown",
      with: [`${ability.cooldown(abilityLevel)}`],
    });
  }
  body.pushLine();
  if (abilityLevel < 5) {
    body.pushLine({
      translate: "pod_rpg.common.ui.ability.level",
      with: [
        `${abilityLevel + 1}`,
        `${5 + abilityLevel * 5}`,
        `${player.level}`,
      ],
    });
  }
  body.pushLine();
  for (let i = 1; i < 6; i++) {
    const unlocked = abilityLevel >= i;
    body.pushLine(
      { text: unlocked ? "§7[§a" : "§8[§c" },
      { translate: "pod_rpg.common.ui.level_text", with: [`${i}`] },
      { text: unlocked ? "§7] §r" : "§8] §r" },
      { translate: `pod_rpg.abilities.${ability.key}.level${i}` }
    );
  }
  body.pushLine();

  if (level >= ability.unlock_level) {
    const selected = getSelectedAbility(player, skill)?.key === ability.key;
    buttons.push({
      translation_key: `pod_rpg.common.ui.ability.${
        selected ? "disable_button" : "enable_button"
      }`,
      icon_path: `textures/podcrash/rpg/icons/abilities/${
        selected ? "gray/" : ""
      }${ability.key}.png`,
      action: () => {
        setSelectedAbility(
          player,
          skill,
          selected ? -1 : skill.abilities.indexOf(ability)
        );
        showSkillInfoPage(player, skill);
      },
    });

    if (abilityLevel < 5 && player.level >= 5 + abilityLevel * 5) {
      buttons.push({
        translation_key: "pod_rpg.common.ui.ability.level_button",
        action: () => {
          player.addLevels(-(5 + abilityLevel * 5));
          player.playSound("random.levelup");
          player.setDynamicProperty(
            `pod_rpg_${skill.key}_${ability.key}`,
            abilityLevel + 1
          );
          showAbilityPage(player, skill, ability);
        },
      });
    }
  } else {
    body.pushLine(
      { text: "§c" },
      { translate: "pod_rpg.common.ui.ability.not_unlocked" }
    );
  }

  buttons.push({
    translation_key: "pod_rpg.common.ui.back_button",
    action: () => {
      showSkillInfoPage(player, skill);
    },
  });

  showPage(
    player,
    { rawtext: [{ translate: `pod_rpg.abilities.${ability.key}.name` }] },
    body.build(),
    buttons
  );
};
