import { system } from "@minecraft/server";
import { displayActionBar } from "./utils";
import { getAbilityName, getSelectedAbility } from "./addonUtils";

export default class TimerUtil {
  constructor() {
    this.cooldowns = new Map();
    this.onExpire = {};
    this.displayPriority = 2;
  }

  has(id) {
    return (
      this.cooldowns.has(id) && this.cooldowns.get(id) > system.currentTick
    );
  }

  get(id) {
    return this.cooldowns.get(id) || 0;
  }

  set(id, ticks) {
    const runId = this.onExpire[id];
    if (runId !== undefined) system.clearRun(runId);
    this.cooldowns.set(id, system.currentTick + ticks);
  }

  enableDisplay(player, text) {
    const runId = system.runInterval(() => {
      if (!this.has(player.name)) {
        displayActionBar(player, this.displayPriority, "§r");
        system.clearRun(runId);
        return;
      }
      displayActionBar(player, this.displayPriority, text());
    }, 5);
  }

  setOnExpire(id, func) {
    if (!this.has(id)) return;
    const runId = system.runTimeout(() => {
      this.onExpire[id] = undefined;
      func();
    }, this.get(id) - system.currentTick);
    this.onExpire[id] = runId;
  }
}

export class AbilityTimerUtil extends TimerUtil {
  set(player, ticks, skill) {
    const ability = getSelectedAbility(player, skill);
    if (ability === undefined) return;
    super.set(player.id + ability.key, ticks);
    if (skill === undefined) return;
    super.setOnExpire(player.name, () => {
      displayActionBar(player, 1, {
        rawtext: [
          ...getAbilityName(
            getSelectedAbility(player, skill),
            skill.secondary_color
          ),
          { text: " " },
          { translate: "pod_rpg.common.ability.cooldown.expire" },
        ],
      });
    });
  }
}
