import { world } from "@minecraft/server";
import { MessageFormData, ModalFormData } from "@minecraft/server-ui";
import { showConfirmationScreen } from "./uiUtils";

const settings = [
  {
    id: "reduce_validations",
    type: "boolean",
    default: false,
    value: false,
  },
  {
    id: "addon_integration",
    type: "boolean",
    default: true,
    value: true,
  },
  {
    id: "particles",
    type: "boolean",
    default: true,
    value: true,
  },
  {
    id: "double_jump",
    type: "boolean",
    default: true,
    value: true,
  },
  {
    id: "slower_double_click",
    type: "boolean",
    default: false,
    value: false,
  },
  {
    id: "explorer_exp_bar",
    type: "boolean",
    default: true,
    value: true,
  },
  {
    id: "explosion_damage",
    type: "boolean",
    default: true,
    value: true,
  },
  {
    id: "fire_damage",
    type: "boolean",
    default: true,
    value: true,
  },
  {
    id: "ability_unlock_message",
    type: "boolean",
    default: true,
    value: true,
  },
  {
    id: "announce_level_up",
    type: "boolean",
    default: true,
    value: true,
  },
  {
    id: "abilities_target_players",
    type: "boolean",
    default: true,
    value: true,
  },
];

export const getSetting = (settingId) => {
  const setting = settings.find((v) => v.id === settingId);
  if (setting === undefined) return undefined;
  return setting.value !== undefined ? setting.value : setting.default;
};

export const loadSettings = () => {
  for (const setting of settings) {
    const value = world.getDynamicProperty("pod_rpg_setting_" + setting.id);
    setting.value = value === undefined ? setting.default : value;
  }
};

export const showSettingsDisclaimer = (player) => {
  const form = new MessageFormData();
  form.title({ rawtext: [{ translate: "pod_rpg.settings.title" }] });
  form.body({ rawtext: [{ translate: "pod_rpg.settings.disclaimer" }] });
  form.button1({
    rawtext: [{ translate: "pod_rpg.common.ui.cancel_button" }],
  });
  form.button2({
    rawtext: [{ translate: "pod_rpg.common.ui.continue_button" }],
  });
  showConfirmationScreen(player, form, () => showSettingsMenu(player));
};

export const showSettingsMenu = async (player) => {
  const form = new ModalFormData();
  form.title({ rawtext: [{ translate: "pod_rpg.settings.title" }] });
  const listedSettings = [];
  for (const setting of settings) {
    const value = getSetting(setting.id);
    if (value === undefined) continue;
    switch (setting.type) {
      case "boolean":
        form.toggle(
          {
            rawtext: [{ translate: `pod_rpg.setting.${setting.id}` }],
          },
          value
        );
        listedSettings.push(setting);
        break;
    }
  }

  const response = await form.show(player);
  if (!response.formValues) return;
  for (let i = 0; i < listedSettings.length; i++) {
    const value = response.formValues[i];
    if (value === undefined) continue;
    world.setDynamicProperty("pod_rpg_setting_" + listedSettings[i].id, value);
  }
  loadSettings();
};
