import { ActionFormData } from "@minecraft/server-ui";
import { log } from "./utils";

/*
const buttonExample = {
  translation_key: "pod_farm.guide.main.button.0",
  icon_path: "textures/items/iron_ingot", //can be undefined, button will have no icon then.
  action: showPage("main", 0, []), //can be any function or undefined. If undefined, the button will close the menu.
};
*/

export const showConfirmationScreen = (player, form, onSuccess) => {
  form.show(player).then((response) => {
    if (response.canceled || response.selection !== 1) return;
    onSuccess();
  });
};

export const showTextPage = (player, pageName, entries, buttons) => {
  const body = new RawtextBuilder();
  for (let i = 0; i < entries; i++) {
    body.pushLine({
      translate: `pod_farm.guide.${pageName}.${i}`,
      with: ["\n"],
    });
  }
  showPage(
    player,
    { rawtext: [{ translate: `pod_farm.guide.${pageName}.title` }] },
    body.build(),
    buttons
  );
};

export const showPage = (player, title, body, buttons) => {
  if (!buttons || !buttons.length) {
    log(`Failed to show Page: Requires Button.`);
    return;
  }
  const form = new ActionFormData();
  form.title(title);
  form.body(body);
  for (const button of buttons) {
    form.button(
      { rawtext: [{ translate: button.translation_key }] },
      button.icon_path
    );
  }
  form.show(player).then((response) => {
    if (response.canceled || response.selection === undefined) return;
    const action = buttons[response.selection].action;
    if (!action) return;
    action();
  });
};

export class RawtextBuilder {
  constructor() {
    this.body = [];
  }

  push(...elements) {
    for (const element of elements) {
      this.body.push(element);
    }
  }

  pushLine(...elements) {
    this.push(...elements);
    this.push({ text: "§r\n" });
  }

  build() {
    return { rawtext: this.body };
  }
}
