import { world } from "@minecraft/server";
import { spawnParticle } from "../../utils/utils";

export const blast_shield_ability = {
  key: "blast_shield",
  unlock_level: 15,
  enableTitle: true,
  description: () => {
    return [];
  },
  cooldown: (level) => getCooldown(level),
  onEnable: (data) => {
    const { player, skill, level } = data;
    const entities = player.dimension.getEntities({
      location: player.location,
      maxDistance: 5,
      excludeTypes: ["item", "xp_orb"],
    });
    for (const entity of entities) {
      if (!entity?.location || entity === player || !entity.isValid()) continue;
      entity.applyKnockback(
        entity.location.x - player.location.x,
        entity.location.z - player.location.z,
        getKnockbackStrength(level),
        getKnockbackStrength(level) / 5
      );
    }
    spawnParticle(player.dimension, player.location, "pod_rpg:blastshield");
    world.playSound("random.explode", player.location);
    skill.abilityCooldown.set(player, getCooldown(level) * 20, skill);
  },
};

const getKnockbackStrength = (level) => {
  let value = 2;
  if (level >= 2) value += 1;
  if (level >= 4) value += 1;
  return value;
};

const getCooldown = (level) => {
  let cooldown = 60;
  if (level >= 1) cooldown -= 10;
  if (level >= 3) cooldown -= 10;
  if (level >= 5) cooldown -= 10;
  return cooldown;
};
